#!/bin/bash

#    devparanoia - Terminal-interactive device selector.
#    Copyright (C) 2016 David B. Hedlund <public@beloved.name>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

require_drive_without_root_partition_mounted=true;
require_removable_device=true;
require_unmount=true;

clear

echo "
Usage example:

Just list the selected device
./devparanoia.sh && source devparanoia_output && ls /dev/\$dev; rm -f devparanoia_output

Write an image to the device
./devparanoia.sh && source devparanoia_output && sudo dd of=/dev/\$dev if=\"/home/you/image.iso\" bs=4M conv=noerror,sync; rm -f devparanoia_output
"

# show menu
echo "---------------------------------"
echo "Device selector

RM=removable device"
echo "---------------------------------"

bvar=0;
for b in $(lsblk -d -o NAME -n); do
    ((bvar++))
    if [ "$bvar" = 1 ]; then echo "#  $(lsblk /dev/$b -d -o NAME,SIZE,TYPE,MODEL,RM)" | grep NAME; fi
    echo "$bvar: $(lsblk /dev/$b -d -n -o NAME,SIZE,TYPE,MODEL,RM)"
    
done
echo "---------------------------------"

while :
do
    read -r -p "Select a device number [1-$bvar]: " opt
    # take action
    case $opt in
	[1-$bvar])

	    for x in $(lsblk -d -o NAME -n); do
		((cvar++))
		if [ "$opt" == "$cvar" ]; then dev=$x; fi
	    done

	    if [ "$require_drive_without_root_partition_mounted" = true ]; then

		# Root partition device check
		if [ "$(findmnt | grep /dev/$dev | colrm 2)" == / ]; then write=false; echo "WARNING: $dev contains the mounted root partition of this system!"; exit
		else echo "OK: $dev is not a system device.";
		fi

	    fi

	    if [ "$require_removable_device" = true ]; then

		# Removable device check
		# Must be trimmed
		if [ "$(lsblk /dev/$dev -d -n -o RM | xargs)" == 0 ]; then write=false; echo "WARNING: $dev is not a removable device!"; exit; else echo "OK: $dev is a removable device."; fi

	    fi

	    # Mount check - requires sudo so added last

	    if [ "$require_unmount" = true ]; then
		if [ "$(mount | grep $dev)" ]; then echo "Requesting umount permission" && sudo umount /dev/$dev?; fi
		if [ "$(mount | grep $dev)" ]; then write=false; echo "WARNING: Could not unmount $dev. Never use dd not write to a mounted device."; exit; else echo "OK: $dev is not mounted."; fi
	    fi

	    echo "dev=$dev" > devparanoia_output && echo -e "\ncat devparanoia_output" && cat devparanoia_output;
	    unset require_drive_without_root_partition_mounted require_unmount bvar cvar opt write
	    
	    exit
	    ;;
    esac
done
